/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/Table.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron;
import openjava.mop.OJClass;
import openjava.ptree.Expression;
import saffron.opt.Cluster;
import saffron.rel.Rel;

/**
 * A <code>Table</code> is a relational dataset in a {@link saffron.Schema}.  It has
 * methods to describe and implement itself.
 *
 * @author jhyde
 * @since 10 November, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/Table.java#2 $
 **/
public interface Table {
	/**
	 * Returns the {@link saffron.Schema} this table belongs to.
	 **/
	Schema getSchema();

	/**
	 * Describes the type of rows returned by this table.
	 **/
	OJClass getRowType();

	/**
	 * Returns an estimate of the number of rows in the table.
	 **/
	double getRowCount();

	/**
	 * Converts this table into a {@link saffron.rel.Rel}.  Rules may later transform this
	 * relational expression into other access methods for this table.
	 *
	 * @param cluster the cluster this Rel will belong to
	 * @param schemaExp the parse tree of the expression which evaluates to a
	 *        schema object
	 **/
	Rel toRel(Cluster cluster, Expression schemaExp);
}


// End Table.java
