/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/JdbcConnection.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.ext;
import saffron.Connection;
import saffron.Schema;

/**
 * A <code>JdbcConnection</code> is an implementation of {@link
 * saffron.Connection} which gets its data from a JDBC database.
 *
 * <p>Derived classes must implement {@link #getSchema} and {@link
 * #getSchemaStatic}.</p>
 *
 * @author jhyde
 * @since 10 November, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/JdbcConnection.java#2 $
 **/
public abstract class JdbcConnection implements saffron.Connection
{
	java.sql.Connection sqlConnection;

	public JdbcConnection(java.sql.Connection sqlConnection)
	{
		this.sqlConnection = sqlConnection;
	}

	public java.sql.Connection getConnection()
	{
		return sqlConnection;
	}

	// implement Connection
	public Object contentsAsArray(String tableName)
	{
		throw new UnsupportedOperationException(
			"JdbcConnection.contentsAsArray(" + tableName +
			") should have been replaced");
	}

	// for Connection
	public static Schema getSchemaStatic()
	{
		throw new UnsupportedOperationException(
			"Derived class must implement " +
			"public static Schema getSchemaStatic()");
	}
}

// End JdbcConnection.java
