/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/ObjectTable.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.ext;

import openjava.mop.OJClass;
import openjava.ptree.Expression;
import saffron.Schema;
import saffron.Table;
import saffron.opt.Cluster;
import saffron.rel.Rel;

/**
 * <code>ObjectTable</code> is a component of an {@link ObjectSchema}.
 *
 * @author jhyde
 * @since 8 February, 2002
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/ObjectTable.java#1 $
 **/
public class ObjectTable implements Table
{
	Schema schema;
	OJClass rowType;

	public ObjectTable(Schema schema, OJClass rowType)
	{
		this.schema = schema;
		this.rowType = rowType;
	}

	public ObjectTable(Schema schema, Class rowType)
	{
		this(schema, OJClass.forClass(rowType));
	}

	// implement Table
	public Schema getSchema()
	{
		return schema;
	}
	
	// implement Table
	public OJClass getRowType()
	{
		return rowType;
	}
	
	public double getRowCount()
	{
		return 100;
	}

	// implement Table
	public Rel toRel(Cluster cluster, Expression connectionExp)
	{
		throw new UnsupportedOperationException();
	}

	/**
	 * Records the fact that there is another way to access that a field access
	 * or method call. For example, "select from classes as c where
	 * c.getName().equals(x)" is the same as "schema.classForName(x)".
	 **/
	public void addAccessor(String fieldName, String schemaMethodName)
	{}
}


// End ObjectTable.java
