/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/Operand.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.opt;

import saffron.util.Walkable;

/**
 * An <code>Operand</code> determines whether a {@link saffron.opt.Rule} can be
 * applied to a particular expression.
 *
 * <p>For example, the rule to pull a filter up from the left side of a
 * join takes operands: <code>(Join (Filter) (Any))</code>.</p>
 *
 * <p>Note that <code>children</code> means different things if it is empty
 * or it is <code>null</code>: <code>(Join (Filter <b>()</b>) (Any))</code>
 * means that, to match the rule, <code>Filter</code> must have no
 * operands.
 **/
public class Operand implements Walkable
{
    Class clazz;
    Operand[] children;
    Operand parent;
    int ordinalInParent;
    Rule rule;
    int ordinalInRule;
    int[] solveOrder;

    public Operand(Class clazz, Operand[] children)
    {
        this.clazz = clazz;
        this.children = children;
        if (children != null) {
            for (int i = 0; i < this.children.length; i++) {
                this.children[i].parent = this;
            }
        }
    }

    // implement Walkable
    public Object[] getChildren()
    {
        return children;
    }
}
