/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/Query.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.opt;
import saffron.rel.Rel;

import java.util.HashMap;

/**
 * A <code>Query</code> represents a set of {@link saffron.rel.Rel}s which come from the
 * same <code>select</code> statement.
 **/
public class Query {
	private int nextCorel = 0;
	/**
	 * Maps name of correlating variable (e.g. "$Corel3") to the {@link Rel}
	 * which implements it.
	 */
	HashMap mapCorelToRel;
	/**
	 * Maps a from-list expression to the name of the correlating variable
	 * which references it. This is for forward-references, caused when from
	 * items have correlating variables. We will later resolve to a
	 * {@link Rel}.
	 */
	public HashMap mapDeferredToCorel;

	public Query()
	{
		mapCorelToRel = new HashMap();
		mapDeferredToCorel = new HashMap();
	}
	/**
	 * Constructs a new name for a correlating variable.  It is unique within
	 * the whole query.
	 **/
	public String createCorel() {
		int n = nextCorel++;
		return "$corel" + n;
	}

	/**
	 * Maps a correlating variable to a {@link Rel}.
	 */
	public void mapCorel(String name, Rel rel) {
		mapCorelToRel.put(name, rel);
	}

	/**
	 * Creates a name for a correlating variable for which no {@link Rel} has
	 * been created yet.
	 *
	 * @param deferredLookup contains the information required to resolve the
	 *     variable later
	 */
	public String createCorelUnresolved(Object deferredLookup)
	{
		int n = nextCorel++;
		String name = "$corel" + n;
		mapDeferredToCorel.put(deferredLookup, name);
		return name;
	}
	/**
	 * Returns the relational expression which populates a correlating
	 * variable.
	 **/
	public Rel lookupCorel(String name)
	{
		return (Rel) mapCorelToRel.get(name);
	}
}

// End Query.java
