/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/Rule.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.opt;



/**
 * A <code>Rule</code> transforms an expression into another.
 **/
public abstract class Rule
{
    /**
     * Current planner; set by {@link VolcanoPlanner#VolcanoPlanner}.
     **/
    protected VolcanoPlanner planner;
    /** Root of operand tree. */
    Operand operand;
    /** Flattened list of operands. */
    Operand[] operands;

    Rule(Operand operand)
    {
        this.operand = operand;
    }

    /**
     * Return the current {@link VolcanoPlanner}.
     */
    VolcanoPlanner getPlanner()
    {
        return planner;
    }

    /**
     * Return the calling convention which the generated expression will
     * support.
     */
    public abstract int getConvention();

	/**
	 * This method is called every time the rule matches. At the time that this method is called, {@link RuleCall#rels
	 * call.rels} holds the set of relational expressions which match the
	 * operands to the rule; <code>call.rels[0]</code> is the root
     * expression.
	 *
	 * <p> Typically a rule would call {@link RuleCall#transformTo}, to produce
	 * an expression whose output matches <code>convention</code>.
	 */
	public abstract void onMatch(RuleCall call);

    /**
     * Return a description of this rule.
     */
    String getDescription() {
        return getClass().toString();
    }
}

// End Rule.java
