/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/AggUnpickler.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;

import openjava.ptree.Expression;
import openjava.ptree.Variable;
import openjava.ptree.util.EvaluationShuttle;
import saffron.util.RelEnvironment;
import saffron.util.Util;

/**
 * Converts references to group items and aggregates (created by {@link
 * AggInternalTranslator}) into more conventional field accesses.
 **/
class AggUnpickler extends EvaluationShuttle
{
    int groupCount;

    AggUnpickler(QueryInfo queryInfo, int groupCount)
    {
        super(queryInfo.env);
        this.groupCount = groupCount;
    }
    public Expression evaluateDown(Variable variable)
    {
        String name = variable.toString();
        if (name.startsWith("$group")) {
            int i = Integer.parseInt(name.substring("$group".length()));
            return RelEnvironment.makeFieldAccess(0, i);
        } else if (name.startsWith("$agg")) {
            int i = Integer.parseInt(name.substring("$agg".length()));
            return RelEnvironment.makeFieldAccess(0, groupCount + i);
        } else if (name.startsWith("$input")) {
            throw Util.newInternal(
                "Expression " + variable + " is neither a constant nor " +
                "an aggregate function");
        } else {
            // Other variables -- from the enclosing Java environment, or
            // correlating variables such as $corel2 -- are OK.
            return variable;
        }
    }
}
