/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/Correlator.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;
import openjava.ptree.Expression;
import openjava.ptree.Literal;
import saffron.Planner;
import saffron.opt.Cluster;
import saffron.util.Util;

/**
 * A <code>Correlator</code> behaves like a kind of {@link Join}, but works by
 * setting variables in its environment and restarting its right-hand input.
 * It is used to represent a correlated query; one implementation option is to
 * de-correlate the expression.
 *
 * @author jhyde
 * @since 23 September, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/Correlator.java#1 $
 **/
public class Correlator extends Join {

	Expression[] correlations;

	/**
	 * Creates a Correlator.
	 *
	 * @param cluster     {@link Cluster} this relational expression belongs to
	 * @param left      left input relational expression
	 * @param right     right  input relational expression
	 * @param joinType  join type (see {@link Join#joinType})
	 * @param correlations set of expressions to set as variables each time a
	 *                  row arrives from the left input
	 **/
	Correlator(
		Cluster cluster, Rel left, Rel right, int joinType,
		Expression[] correlations)
	{
		super(cluster, left, right, Literal.constantTrue(), joinType);
		this.correlations = correlations;
	}

	public Object clone()
	{
		return new Correlator(
			cluster, (Rel) left.clone(), (Rel) right.clone(), joinType,
			Util.clone(correlations));
	}

	// override Rel
	public Rel changeConvention(Planner planner, int convention)
	{
		switch (convention) {
//  		case CallingConvention.INLINE:
//  			return new saffron.rel.java.JavaFilter(
//  				cluster, planner.convert(child, convention), condition);
//  		case CallingConvention.PLAN:
//  			return new PlanRel.PlanFilter(
//  				cluster, planner.convert(child, convention), condition);
		default:
			return null;
		}
	}
}


// End Correlator.java
