/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/Distinct.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;
import openjava.mop.OJClass;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.rel.java.JavaDistinct;
import saffron.util.Util;

/**
 * <code>Distinct</code> is a {@link saffron.rel.Rel} which eliminates duplicates from its
 * input.
 *
 * @author jhyde
 * @since 30 September, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/Distinct.java#1 $
 **/
public class Distinct extends SingleRel {
	public Distinct(Cluster cluster, Rel child)
	{
		super(cluster, child);
	}
	
	public Object clone() {
		return new Distinct(cluster, Util.clone(child));
	}

	// implement Rel
	protected OJClass deriveRowType()
	{
		return child.getRowType();
	}

	// implement Plan
	public Cost computeSelfCost(Planner planner)
	{
		return planner.makeZeroCost();
	}

	// override Rel
	public Rel changeConvention(Planner planner, int convention)
	{
		switch (convention) {
		case CallingConvention.JAVA:
			Rel convertedChild = planner.convert(child, convention);
			if (convertedChild == null) {
				return null;
			}
			return new JavaDistinct(cluster, convertedChild);
		default:
			return null;
		}
	}
}

// End Distinct.java
