/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/DistinctToExistsRule.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;

import openjava.mop.OJClass;
import openjava.ptree.util.SyntheticClass;
import saffron.opt.CallingConvention;
import saffron.opt.Operand;
import saffron.opt.RuleCall;
import saffron.opt.SingleConventionRule;
import saffron.rel.java.JavaDistinct;
import saffron.rel.java.JavaExists;

/**
 * Rule to translate a {@link saffron.rel.java.JavaDistinct} into a {@link
 * saffron.rel.java.JavaExists}, provided that the select list contains zero
 * columns.
 **/
public class DistinctToExistsRule extends SingleConventionRule
{
    public DistinctToExistsRule()
    {
        super(
            new Operand(JavaDistinct.class, null),
            CallingConvention.JAVA);
    }

    public void onMatch(RuleCall call)
    {
        JavaDistinct distinct = (JavaDistinct) call.rels[0];
        OJClass rowType = distinct.child.getRowType();
        if (rowType instanceof SyntheticClass &&
            ((SyntheticClass) rowType).getWidth() == 0) {
            call.transformTo(new JavaExists(
                distinct.cluster, distinct.child));
        }
    }
}

// End DistinctToExistsRule.java
