/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/NestedLoopJoinRule.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;

import saffron.opt.CallingConvention;
import saffron.opt.Operand;
import saffron.opt.RuleCall;
import saffron.opt.SingleConventionRule;

/**
 * Rule to translate a {@link Join} into a {@link
 * saffron.rel.java.JavaNestedLoopJoin}.  It does so by calling {@link
 * Join#changeConvention}.
 **/
public class NestedLoopJoinRule extends SingleConventionRule
{
    public NestedLoopJoinRule(int convention)
    {
        super(
            new Operand(Join.class, null),
            convention);
    }

	public void onMatch(RuleCall call) {
        Join join = (Join) call.rels[0];
        if (join.getClass() == Join.class) {
            // We can translate a Join into an JavaNestedLoopJoin, but if
            // we allow sub-types, we end up translating an
            // JavaNestedLoopJoin into a copy of itself, forever.
			Rel converted = join;
			if (convention != CallingConvention.NONE) {
				converted = join.changeConvention(planner, convention);
			}
			if (converted != null) {
				call.transformTo(converted);
			}
		}
    }
}

// End NestedLoopJoinRule.java
