/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/RemoveTrivialProjectRule.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;

import openjava.ptree.Variable;
import saffron.opt.*;
import saffron.util.RelEnvironment;

/**
 * <code>RemoveTrivialProjectRule</code> removes {@link Project} nodes
 * which merely return their input. For example,
 * <code>Project(ArrayReader(a), {$input0})</code> becomes
 * <code>ArrayReader(a)</code>.
 **/
public class RemoveTrivialProjectRule extends SingleConventionRule
{
    public RemoveTrivialProjectRule()
    {
        super(
            new Operand(Project.class, null),
            CallingConvention.NONE);
    }

	public void onMatch(RuleCall call) {
        Project project = (Project) call.rels[0];
        if (project.exps.length == 1 &&
            project.exps[0] instanceof Variable &&
            RelEnvironment.getInputOrdinal(
                project.exps[0].toString()) == 0) {
            Rel rel = project.child;
            RelSubset subset = planner.register(
                rel, planner.getSet(project),
                VolcanoPlanner.MAY_BE_REGISTERED);
            call.transformTo(planner.convert(subset, CallingConvention.NONE));
        }
    }
}

// End RemoveTrivialProjectRule.java
