/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/SingleRel.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;
import openjava.mop.OJClass;
import saffron.opt.Cluster;
import saffron.util.Util;

/**
 * A <code>SingleRel</code> is a base class single-input relational
 * expressions.
 *
 * @author jhyde
 * @since 23 September, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/SingleRel.java#1 $
 **/
public abstract class SingleRel extends Rel {
	public Rel child;

	/**
	 * Creates a <code>SingleRel</code>.
	 *
	 * @param cluster     {@link Cluster} this relational expression belongs to
	 * @param child     input relational expression
	 **/
	protected SingleRel(Cluster cluster, Rel child)
	{
		super(cluster);
		this.child = child;
	}

	// implement Rel
	public Rel[] getInputs()
	{
		return new Rel[] {child};
	}

	// override Rel
	public void replaceInput(int ordinalInParent, Rel rel)
	{
		Util.assert(ordinalInParent == 0);
		this.child = rel;
	}

	// implement Rel
	protected OJClass deriveRowType()
	{
		return child.getRowType();
	}
}


// End SingleRel.java
