/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/UnionToDistinctRule.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;

import saffron.opt.Operand;
import saffron.opt.RuleCall;
import saffron.opt.SingleConventionRule;

/**
 * <code>UnionToDistinctRule</code> translates a distinct {@link Union}
 * (<code>all</code> = <code>false</code>) into a {@link Distinct} on top
 * of a non-distinct {@link Union} (<code>all</code> = <code>true</code>).
 **/
public class UnionToDistinctRule extends SingleConventionRule
{
    public UnionToDistinctRule()
    {
        super(
            new Operand(Union.class, null),
            saffron.opt.CallingConvention.NONE);
    }

	public void onMatch(RuleCall call) {
        Union union = (Union) call.rels[0];
        if (union.getClass() != Union.class) {
            return; // require precise class, otherwise we loop
        }
        if (union.all) {
            return; // nothing to do
        }
        Union unionAll = new Union(union.cluster, union.inputs, true);
        call.transformTo(new Distinct(union.cluster, unionAll));
    }
}

// End UnionToDistinctRule.java
