/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/convert/IterConverter.java#3 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.convert;

import openjava.mop.OJClass;
import openjava.ptree.*;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.util.RelEnvironment;
import saffron.util.UnboundVariableCollector;
import saffron.util.Util;

/**
 * <code>IterConverter</code> converts a plan from <code>inConvention</code> to
 * {@link saffron.opt.CallingConvention#ITERATOR}.
 **/
class IterConverter extends Converter
{
    IterConverter(
        Cluster cluster, Rel child, int inConvention)
    {
        super(cluster, child, inConvention);
    }

    public Object clone()
    {
        return new IterConverter(this.cluster, child, inConvention);
    }

    // implement Rel
    public int getConvention()
    {
        return CallingConvention.ITERATOR;
    }

    public static boolean canConvert(int inConvention)
    {
        switch (inConvention) {
        case CallingConvention.ENUMERATION:
        case CallingConvention.ITERABLE:
        case CallingConvention.COLLECTION:
            return true;
        default:
            return false;
        }
    }

    // implement Converter
    protected Object implementEnumeration(
        Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   new saffron.runtime.EnumerationIterator(<<child>>)
            Expression exp = (Expression) implementor.implementChild(
                this, 0, child);
            return new AllocationExpression(
                OJClass.forClass(saffron.runtime.EnumerationIterator.class),
                new ExpressionList(exp));
        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }

    // implement Converter
    protected Object implementIterable(
		Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   <<exp>>.iterator()
            Expression exp = (Expression) implementor.implementChild(
                this, 0, child);
            return new MethodCall(exp, "iterator", null);

        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }

    // implement Converter
    public Object implementCollection(
        Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   <<exp>>.iterator()
            Expression exp = (Expression) implementor.implementChild(
                this, 0, child);
            return new MethodCall(exp, "iterator", null);

        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }
}
