/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/convert/IterableConverter.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.convert;

import openjava.mop.OJClass;
import openjava.ptree.*;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.util.RelEnvironment;
import saffron.util.UnboundVariableCollector;
import saffron.util.Util;

/**
 * <code>IterableConverter.java</code> converts a plan from <code>inConvention</code> to
 * {@link saffron.opt.CallingConvention#ITERABLE}.
 **/
class IterableConverter extends Converter
{
    IterableConverter(
        Cluster cluster, Rel child, int inConvention)
    {
        super(cluster, child, inConvention);
    }

    public Object clone()
    {
        return new IterableConverter(this.cluster, child, inConvention);
    }

    // implement Rel
    public int getConvention()
    {
        return CallingConvention.ITERABLE;
    }

    public static boolean canConvert(int inConvention)
    {
        switch (inConvention) {
        case CallingConvention.JAVA:
        case CallingConvention.ENUMERATION:
        case CallingConvention.COLLECTION:
            return true;
        default:
            return false;
        }
    }

    // implement Converter
    protected Object implementJava(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1: // called from parent
            // Generate
            //   new saffron.runtime.ThreadIterator() {
            //     private C0 v0;
            //     ...
            //     public saffron.runtime.ThreadIterator init(C0 v0, ...) {
            //       this.v0 = v0;
            //       ...
            //       return this;
            //     }
            //     protected void doWork() {
            //       <<for each child row o>>
            //          put(<<o>>);
            //       <<end for>>
            //     }
            //   }.init(v0, ...)
            UnboundVariableCollector unboundVars =
                Util.getUnboundVariables(this);
            StatementList body = new StatementList();
            implementor.pushStatementList(body);
            implementor.implementChild(this, 0, child);
            implementor.popStatementList(body);

            // private C0 v0;
            // ...
            MemberDeclarationList memberDeclarationList =
                unboundVars.getMemberDeclarationList();

            // this.v0 = v0;
            // ...
            // return this;
            StatementList initBody = unboundVars.getAssignmentList();
            initBody.add(
                new ReturnStatement(
					SelfAccess.constantThis()));

            memberDeclarationList.add(
                new MethodDeclaration(
                    new ModifierList(ModifierList.PUBLIC),
                    TypeName.forOJClass(
                        OJClass.forClass(
                            saffron.runtime.ThreadIterator.class)),
                    "init",
                    unboundVars.getParameterList(),
                    null,
                    initBody));
            memberDeclarationList.add(
                new MethodDeclaration(
                    new ModifierList(ModifierList.PROTECTED),
                    TypeName.forOJClass(
                        openjava.mop.OJSystem.VOID),
                    "doWork",
                    new ParameterList(),
                    null,
                    body));
            return new MethodCall(
                new AllocationExpression(
                    TypeName.forOJClass(
                        OJClass.forClass(
                            saffron.runtime.ThreadIterator.class)),
                    new ExpressionList(),
                    memberDeclarationList),
                "init",
                unboundVars.getArgumentList());

        case 0:
            StatementList stmtList = implementor.getStatementList();
            stmtList.add(
                new ExpressionStatement(
                    new MethodCall(
                        (Expression) null,
                        "put",
                        new ExpressionList(
                            implementor.translate(
                                this, RelEnvironment.makeReference(0))))));
            return null;

        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);

        }
    }

    // implement Converter
    protected Object implementIterator(
        Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   new saffron.runtime.BufferedIterator(<<child>>)
            Expression exp = (Expression) implementor.implementChild(
                this, 0, child);
            return new AllocationExpression(
                OJClass.forClass(saffron.runtime.BufferedIterator.class),
                new ExpressionList(exp));
        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }

    // implement Converter
    public Object implementCollection(
        Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   <<exp>>
			//
			// (which is not of the same type, but it should work, because
			// they're just going to call 'iterator' on it)
            return implementor.implementChild(
                this, 0, child);

        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }
}
