/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/ForTerminator.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.ptree.*;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.opt.CallingConvention;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * A <code>ForTerminator</code> takes a {@link saffron.rel.Rel} of {@link
 * CallingConvention#JAVA} and makes it into a <code>for ...</code> loop.
 **/
public class ForTerminator extends SingleTerminator
{
    Variable variable;
    StatementList body;
    String label;

    public ForTerminator(
        Cluster cluster, Rel child, Variable variable, StatementList body,
        String label)
    {
        super(cluster, child);
		Util.assert(child.getConvention() == CallingConvention.JAVA);
        this.variable = variable;
        this.body = body;
        this.label = label;
    }

    // implement Rel
    public Object clone()
    {
        return new ForTerminator(
            cluster, Util.clone(child), (Variable) Util.clone(variable),
            Util.clone(body), label);
    }

    public Object implement(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1: // called from parent
        {
            implementor.setExitStatement(
                new BreakStatement(label));
            StatementList stmtList = new StatementList();
            implementor.pushStatementList(stmtList);
            Object o = implementor.implementChild(this, 0, child);
            Util.assert(o == null);
            implementor.popStatementList(stmtList);
            return stmtList;
        }
        case 0: // called from child
        {
            // Generate
            //   Rowtype variable = <<child variable>>;
            //   <<parent body (references variable)>>
            StatementList stmtList = implementor.getStatementList();
            Expression exp = implementor.translateInput(this, 0);
            stmtList.add(
                new VariableDeclaration(
                    TypeName.forOJClass(child.getRowType()),
                    variable.toString(),
                    exp));
            stmtList.addAll(body);
            return null;
        }
        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }
}
