/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/JavaDistinct.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.mop.Toolbox;
import openjava.ptree.*;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.opt.Implementor;
import saffron.rel.Distinct;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * <code>JavaDistinct</code> implements {@link Distinct} inline. See also
 * {@link JavaAggregate}.
 **/
public class JavaDistinct extends Distinct
{
	Variable var_h;

	public JavaDistinct(Cluster cluster, Rel child)
	{
		super(cluster, child);
	}

	// implement Rel
	public Object clone()
	{
		return new JavaDistinct(cluster, child);
	}

	public int getConvention()
	{
		return CallingConvention.JAVA;
	}

	public Cost computeSelfCost(Planner planner)
	{
		double dRows = child.getRows(),
			dCpu = Util.nLogN(dRows), // reflects memory cost also
			dIo = 0;
		return planner.makeCost(dRows, dCpu, dIo);
	}

	public Object implement(Implementor implementor, int ordinal)
	{
		switch (ordinal) {
		case -1:
		{
			// Generate
			//	   HashSet h = new HashSet();
			//	   <<parent>>
			StatementList stmtList = implementor.getStatementList();
			this.var_h = implementor.newVariable();
			stmtList.add(
				new VariableDeclaration(
					new TypeName("java.util.HashSet"),
					var_h.toString(),
					new AllocationExpression(
						new TypeName("java.util.HashSet"),
						null)));
			return implementor.implementChild(this, 0, child);
		}
		case 0: // called from child
		{
			// Generate
			//	 if (h.add(<<child variable>>)) {
			//	   <<parent-handler>>
			//	 }
			StatementList stmtList = implementor.getStatementList();
			StatementList ifBody = new StatementList();
			stmtList.add(
				new IfStatement(
					new MethodCall(
						this.var_h,
						"add",
						new ExpressionList(
							Toolbox.box(
								child.getRowType(),
								implementor.translateInput(this, 0)))),
					ifBody));
			implementor.bind(this, child);
			implementor.generateParentBody(this, ifBody);
			return null;
		}
		default:
			throw Util.newInternal(
				"implement: ordinal=" + ordinal);
		}
	}
}

// End JavaDistinct.java
