/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/JavaExists.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.mop.OJClass;
import openjava.mop.Toolbox;
import openjava.ptree.StatementList;
import openjava.ptree.util.SyntheticClass;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.opt.Implementor;
import saffron.rel.Distinct;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * <code>JavaExists</code> implements {@link Distinct} inline for the
 * special case that the input relation has zero columns.
 **/
public class JavaExists extends Distinct
{
    public JavaExists(Cluster cluster, Rel child)
    {
        super(cluster, child);
        OJClass rowType = child.getRowType();
        Toolbox.assert(
            rowType instanceof SyntheticClass &&
            ((SyntheticClass) rowType).getWidth() == 0);
    }

    // implement Rel
    public Object clone()
    {
        return new JavaExists(cluster, child);
    }

    public int getConvention()
    {
        return CallingConvention.JAVA;
    }

    public Cost computeSelfCost(Planner planner)
    {
        // cheaper than JavaDistinct
        double dRows = 1,
            dCpu = dRows,
            dIo = 0;
        return planner.makeCost(dRows, dCpu, dIo);
    }

    public Object implement(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
        {
            // Generate
            //     <<parent>>
            return implementor.implementChild(this, 0, child);
        }
        case 0: // called from child
        {
            // Generate
            //   <<parent-handler>>
            //   break;
            StatementList stmtList = implementor.getStatementList();
            implementor.generateParentBody(this, stmtList);
            stmtList.add(implementor.getExitStatement());
            return null;
        }
        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }
}
