/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/JavaUnionAll.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.ptree.StatementList;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.rel.Union;
import saffron.util.Util;

/**
 * <code>JavaUnionAll</code> implements a {@link Union} inline, without
 * eliminating duplicates.
 **/
public class JavaUnionAll extends Union
{
    public JavaUnionAll(Cluster cluster, Rel[] inputs)
    {
        super(cluster, inputs, true);
    }

    // implement Rel
    public Object clone()
    {
        return new JavaUnionAll(cluster, inputs);
    }

    public int getConvention()
    {
        return CallingConvention.JAVA;
    }

    public Cost computeSelfCost(Planner planner)
    {
        double dRows = getRows(),
            dCpu = 0,
            dIo = 0;
        return planner.makeCost(dRows, dCpu, dIo);
    }

    // Generate
    //   for (int i = 0; i < a.length; i++) {
    //      T row = a[i];
    //      stuff
    //   }
    //   for (int j = 0; j < b.length; j++) {
    //      T row = b[j];
    //      stuff
    //   }
    //
    public Object implement(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            for (int i = 0; i < inputs.length; i++) {
                Util.discard(
                    implementor.implementChild(this, i, inputs[i]));
            }
            return null;

        default:
            if (ordinal >= inputs.length) {
                throw Util.newInternal(
                    "implement: ordinal=" + ordinal);
            }
            // Generate
            //   <<child loop>> {
            //     Type var = <<child row>>
            //     <<parent-handler>>
            //   }
            //   <<next child>>
            StatementList stmtList = implementor.getStatementList();
            implementor.bind(this, inputs[ordinal]);
            implementor.generateParentBody(this, stmtList);
            return null;
        }
    }
}
