/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/util/SaffronSchemaExpander.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.util;
import openjava.mop.Environment;
import openjava.mop.OJClass;
import openjava.ptree.*;
import openjava.ptree.util.VariableBinder;
import saffron.Table;

/**
 * <code>SaffronSchemaExpander</code> converts field accesses of the form
 * <code>schema.field</code> and method calls of the form
 * <code>schema.method(arg0,...)</code>, where <code>schema</code> is an object
 * which implements {@link saffron.Schema} into valid Java fragments like
 * <code>(Sales.Emp[]) schema.contentsAsArray("emps")</code>.  See {@link
 * saffron.Connection#contentsAsArray} for how these fragments are converted
 * into efficient code.
 *
 * @author jhyde
 * @since 11 November, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/util/SaffronSchemaExpander.java#2 $
 **/
public class SaffronSchemaExpander extends VariableBinder {
	public SaffronSchemaExpander(Environment env)
	{
		super(env);
	}

	public Expression evaluateDown(FieldAccess p) throws ParseTreeException
	{
		super.evaluateDown(p);
		Expression refexpr = p.getReferenceExpr(); // e.g. "sales"
		if (refexpr != null) {
			Environment env = getEnvironment();
			OJClass reftype = Util.getType(env, refexpr); // e.g. "SalesSchema"
			String name = p.getName(); // e.g. "emps"
			Table table = Util.getTable(reftype, name);
			if (table != null) {
				// (Emp[]) sales.contentsAsArray("emp")
				return new AliasedExpression(
					new CastExpression(
						OJClass.arrayOf(table.getRowType()),
						new MethodCall(
							refexpr,
							"contentsAsArray",
							new ExpressionList(
								Literal.makeLiteral(name)))),
					name);
			}
		}
		return p;
	}
}

// End SaffronSchemaExpander.java
